
/*
 * File FiniteModelMultiSorted.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J8. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J8 only because so is required by the CASC-J8
 * rules, and for no other reason.
 */


#ifndef __FiniteModelMultiSorted__
#define __FiniteModelMultiSorted__

#include "Lib/Allocator.hpp"
#include "Lib/DHMap.hpp"

#include "Kernel/Unit.hpp"
#include "Kernel/Term.hpp"


namespace FMB {

using namespace Lib;
using namespace Kernel;

class FiniteModelMultiSorted {
 CLASS_NAME(FiniteModelMultiSorted);
 USE_ALLOCATOR(FiniteModelMultiSorted);

 DHMap<unsigned,unsigned> _sizes;

public:

 
 FiniteModelMultiSorted(DHMap<unsigned,unsigned> sortSizes);

 
 
 void addConstantDefinition(unsigned f, unsigned res);
 void addFunctionDefinition(unsigned f, const DArray<unsigned>& args, unsigned res); 
 
 void addPropositionalDefinition(unsigned f, bool res);
 void addPredicateDefinition(unsigned f, const DArray<unsigned>& args, bool res); 

 bool isPartial();

 bool evaluate(Unit* unit);
 unsigned evaluateGroundTerm(Term* term);
 bool evaluateGroundLiteral(Literal* literal);

 vstring toString();

private:

 Formula* partialEvaluate(Formula* formula);
 
 bool evaluate(Formula* formula,unsigned depth=0);

 
 
 bool _isPartial;
 DHMap<unsigned,unsigned> _functionCoverage;
 DHMap<unsigned,unsigned> _predicateCoverage;

 DArray<DArray<int>> sortRepr;

 DArray<unsigned> f_offsets;
 DArray<unsigned> p_offsets;
 DArray<unsigned> f_interpretation;
 DArray<unsigned> p_interpretation; 

 
 DHMap<pair<unsigned,unsigned>,Term*> _domainConstants;
 DHMap<Term*,pair<unsigned,unsigned>> _domainConstantsRev;
public:
 Term* getDomainConstant(unsigned c, unsigned srt)
 {
   Term* t;
   pair<unsigned,unsigned> pair = make_pair(c,srt);
   if(_domainConstants.find(pair,t)) return t;
   vstring name = "domCon_"+env.sorts->sortName(srt)+"_"+Lib::Int::toString(c);
   unsigned f = env.signature->addFreshFunction(0,name.c_str()); 
   env.signature->getFunction(f)->setType(new FunctionType(srt));
   t = Term::createConstant(f);
   _domainConstants.insert(pair,t);
   _domainConstantsRev.insert(t,pair);

   return t;
 }
 pair<unsigned,unsigned> getDomainConstant(Term* t)
 {
   pair<unsigned,unsigned> pair;
   if(_domainConstantsRev.find(t,pair)) return pair;
   USER_ERROR("Evaluated to "+t->toString()+" when expected a domain constant, probably a partial model");
 }
 bool isDomainConstant(Term* t)
 {
   return _domainConstantsRev.find(t);
 }

};


} 
#endif
